/*  
    This library decodes Model 3 Battery CAN messages terminating in A or 2.

    This library was written by Jack Rickard of EVtv - http://www.evtv.me copyright 2019
    You are licensed to use this library for any purpose, commercial or private, 
    without restriction.
    
*/


#include "battery.h"
#


void BATTERY::begin(int Port)
{   
  if (Port == 0)canPort = &CAN0;    
    else canPort = &CAN1;
}

	

void BATTERY::gotFrame(CAN_FRAME *frame)
{                                     //This is our CAN interrupt service routine to catch inbound frames

   switch (frame->id)
     {     
        case 0x132:    
          handle132(frame);
          break;
    
        case 0x292:    
          handle292(frame);
          break;
    
        case 0x352:    
          handle352(frame);
          break;
    
        case 0x3D2:    
          handle3D2(frame);
          break;
    
        case 0x401:    
      	  handle401(frame);
          break;      
 
       case 0x72A:    
          handle72A(frame);
          break;      
     }       
}


void BATTERY::handle132(CAN_FRAME *frame)  
{  
    reportedVoltage=(frame->data.s0)*0.01f;
    Amperes=(frame->data.s1)*0.1f -1000;
    rawCurrent =(frame->data.s2)*0.05f -1000;
  
}

void BATTERY::handle292(CAN_FRAME *frame) //State of Charge
{  
    uint64_t value=frame->data.value; //copy entire frame into value

    SOC=((uint16_t)value & 0x3FF)*0.10f;
  
    value >>= 10;          //shift off 10bits to bit 10.
    minSOC=((uint16_t)value & 0x3FF)*0.10f;

    value >>= 10;          //shift off 10bits to bit 20.
    maxSOC=((uint16_t)value & 0x3FF)*0.10f;

    value >>= 10;          //shift off 10bits to bit 30.
    avgSOC=((uint16_t)value & 0x3FF)*0.10f;  
}

void BATTERY::handle352(CAN_FRAME *frame)  //Capacity, current kWh, kWh needed to full
{  
    uint64_t value=frame->data.value; //copy entire frame into value    

    kWCapacity=((uint16_t)value & 0x3FF)*0.10f;
    
    value >>= 10;          //shift off 10bits to bit 10.          
    kWRemaining=((uint16_t)value & 0x3FF)*0.10f;

    value >>= 30;          //shift off 30bits to bit 40   
    kWNeeded=((uint16_t)value & 0x3FF)*0.10f;

}


void BATTERY::handle3D2(CAN_FRAME *frame)  //Lifetime Charge and Discharge
{	
	 lifeDischarge=frame->data.low *0.001f;
   lifeCharge=frame->data.high*0.001f;
}

void BATTERY::handle401(CAN_FRAME *frame) //Individual Cell Voltages
{		  
    cellindex=frame->data.byte[0];
   
    cell[cellindex*3+1] = frame->data.s1*0.0001f;
    cell[cellindex*3+2] = frame->data.s2*0.0001f;
    cell[cellindex*3+3] = frame->data.s3*0.0001f;
   
    averageCell(97); //Sum cell voltages and calculate average
}

float BATTERY::averageCell(float arraysize)
{
  Voltage=0;
  //This routine receives an array size up to 100 and returns an average of that number of elements in the array
  double totalma = 0; //big float to sum a lot of floats
  for (int i = 1; i < arraysize; i++)
    {
      Voltage += cell[i];
    }
  if(Voltage>0)AverageCell = Voltage / (arraysize-1);
  else AverageCell=0;
  return AverageCell;
}


void BATTERY::handle72A(CAN_FRAME *frame)
{     
  char c;
  uint8_t i=0;
  
  if(frame->data.byte[0]==0)
    { 
      c=0;
      SerialNumber=""; 
      for(i=1;i<8;i++)
        {
          c=frame->data.byte[i];
          SerialNumber = String(SerialNumber  + c); 
        }
    }

   if(frame->data.byte[0]==1)
    {
      for(i=1;i<8;i++)
        {
           c=frame->data.byte[i];
     
        SerialNumber= String(SerialNumber  + c); 
        }
    } 
 }
